@@ -62,8 +62,6 @@
 
 #define MAX_ARITHMETIC_TYPES	10
 
-unsigned char _SLarith_Is_Arith_Type [256];
-
 unsigned char _SLarith_Arith_Types[] =
 {
    SLANG_CHAR_TYPE,
@@ -119,8 +117,10 @@ DEFUN_1(copy_char_to_ulong,char,unsigned long)
 # define copy_char_to_long	copy_char_to_int
 # define copy_char_to_ulong	copy_char_to_uint
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_1(copy_char_to_float,char,float)
 DEFUN_1(copy_char_to_double,char,double)
+#endif
 
 #if SIZEOF_INT != SIZEOF_SHORT
 DEFUN_1(copy_uchar_to_short,unsigned char,short)
@@ -138,8 +138,10 @@ DEFUN_1(copy_uchar_to_ulong,unsigned char,unsigned long)
 # define copy_uchar_to_long	copy_uchar_to_int
 # define copy_uchar_to_ulong	copy_uchar_to_uint
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_1(copy_uchar_to_float,unsigned char,float)
 DEFUN_1(copy_uchar_to_double,unsigned char,double)
+#endif
 
 #if SIZEOF_INT != SIZEOF_SHORT
 DEFUN_1(copy_short_to_char,short,char)
@@ -149,16 +151,20 @@ DEFUN_1(copy_short_to_int,short,int)
 DEFUN_1(copy_short_to_uint,short,unsigned int)
 DEFUN_1(copy_short_to_long,short,long)
 DEFUN_1(copy_short_to_ulong,short,unsigned long)
+#if SLANG_HAS_FLOAT
 DEFUN_1(copy_short_to_float,short,float)
 DEFUN_1(copy_short_to_double,short,double)
+#endif
 DEFUN_1(copy_ushort_to_char,unsigned short,char)
 DEFUN_1(copy_ushort_to_uchar,unsigned short,unsigned char)
 DEFUN_1(copy_ushort_to_int,unsigned short,int)
 DEFUN_1(copy_ushort_to_uint,unsigned short,unsigned int)
 DEFUN_1(copy_ushort_to_long,unsigned short,long)
 DEFUN_1(copy_ushort_to_ulong,unsigned short,unsigned long)
+#if SLANG_HAS_FLOAT
 DEFUN_1(copy_ushort_to_float,unsigned short,float)
 DEFUN_1(copy_ushort_to_double,unsigned short,double)
+#endif
 #else
 # define copy_short_to_char	copy_int_to_char
 # define copy_short_to_uchar	copy_int_to_uchar
@@ -167,17 +173,21 @@ DEFUN_1(copy_ushort_to_double,unsigned short,double)
 # define copy_short_to_uint	copy_int_to_int
 # define copy_short_to_long	copy_int_to_long
 # define copy_short_to_ulong	copy_int_to_ulong
+#if SLANG_HAS_FLOAT
 # define copy_short_to_float	copy_int_to_float
 # define copy_short_to_double	copy_int_to_double
+#endif
 # define copy_ushort_to_char	copy_uint_to_char
 # define copy_ushort_to_uchar	copy_uint_to_uchar
 # define copy_ushort_to_int	copy_int_to_int
 # define copy_ushort_to_uint	copy_int_to_int
 # define copy_ushort_to_long	copy_uint_to_long
 # define copy_ushort_to_ulong	copy_uint_to_ulong
+#if SLANG_HAS_FLOAT
 # define copy_ushort_to_float	copy_uint_to_float
 # define copy_ushort_to_double	copy_uint_to_double
 #endif
+#endif
 
 DEFUN_1(copy_int_to_char,int,char)
 DEFUN_1(copy_int_to_uchar,int,unsigned char)
@@ -206,10 +216,12 @@ DEFUN_1(copy_uint_to_ulong,unsigned int,unsigned long)
 # define copy_uint_to_long	copy_int_to_int
 # define copy_uint_to_ulong	copy_int_to_int
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_1(copy_int_to_float,int,float)
 DEFUN_1(copy_int_to_double,int,double)
 DEFUN_1(copy_uint_to_float,unsigned int,float)
 DEFUN_1(copy_uint_to_double,unsigned int,double)
+#endif
 
 #if SIZEOF_INT != SIZEOF_LONG
 DEFUN_1(copy_long_to_char,long,char)
@@ -219,16 +231,18 @@ DEFUN_1(copy_long_to_ushort,long,unsigned short)
 DEFUN_1(copy_long_to_int,long,int)
 DEFUN_1(copy_long_to_uint,long,unsigned int)
 DEFUN_1(copy_long_to_long,long,long)
-DEFUN_1(copy_long_to_float,long,float)
-DEFUN_1(copy_long_to_double,long,double)
 DEFUN_1(copy_ulong_to_char,unsigned long,char)
 DEFUN_1(copy_ulong_to_uchar,unsigned long,unsigned char)
 DEFUN_1(copy_ulong_to_short,unsigned long,short)
 DEFUN_1(copy_ulong_to_ushort,unsigned long,unsigned short)
 DEFUN_1(copy_ulong_to_int,unsigned long,int)
 DEFUN_1(copy_ulong_to_uint,unsigned long,unsigned int)
+#if SLANG_HAS_FLOAT
+DEFUN_1(copy_long_to_float,long,float)
+DEFUN_1(copy_long_to_double,long,double)
 DEFUN_1(copy_ulong_to_float,unsigned long,float)
 DEFUN_1(copy_ulong_to_double,unsigned long,double)
+#endif
 #else
 #define copy_long_to_char	copy_int_to_char
 #define copy_long_to_uchar	copy_int_to_uchar
@@ -249,6 +263,7 @@ DEFUN_1(copy_ulong_to_double,unsigned long,double)
 #define copy_ulong_to_double	copy_uint_to_double
 #endif
 
+#if SLANG_HAS_FLOAT
 DEFUN_1(copy_float_to_char,float,char)
 DEFUN_1(copy_float_to_uchar,float,unsigned char)
 #if SIZEOF_INT != SIZEOF_SHORT
@@ -267,8 +282,18 @@ DEFUN_1(copy_float_to_ulong,float,unsigned long)
 # define copy_float_to_long	copy_float_to_int
 # define copy_float_to_ulong	copy_float_to_uint
 #endif
+
 DEFUN_1(copy_float_to_float,float,float)
-DEFUN_1(copy_float_to_double,float,double)
+
+static void copy_float_to_double (double *y, float *x, unsigned int n) 
+{ 
+   unsigned int i;
+   for (i = 0; i < n; i++) 
+     y[i] = (double) x[i]; 
+}
+
+
+/* DEFUN_1(copy_float_to_double,float,double) */
 
 DEFUN_1(copy_double_to_char,double,char)
 DEFUN_1(copy_double_to_uchar,double,unsigned char)
@@ -290,7 +315,8 @@ DEFUN_1(copy_double_to_ulong,double,unsigned long)
 #endif
 DEFUN_1(copy_double_to_float,double,float)
 DEFUN_1(copy_double_to_double,double,double)
-
+#endif				       /* SLANG_HAS_FLOAT */
+		    
 DEFUN_2(char_to_int,char,int,copy_char_to_int)
 DEFUN_2(char_to_uint,char,unsigned int,copy_char_to_uint)
 #if SIZEOF_INT != SIZEOF_LONG
@@ -300,8 +326,10 @@ DEFUN_2(char_to_ulong,char,unsigned long,copy_char_to_ulong)
 # define char_to_long	char_to_int
 # define char_to_ulong	char_to_uint
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_2(char_to_float,char,float,copy_char_to_float)
 DEFUN_2(char_to_double,char,double,copy_char_to_double)
+#endif
 
 DEFUN_2(uchar_to_int,unsigned char,int,copy_uchar_to_int)
 DEFUN_2(uchar_to_uint,unsigned char,unsigned int,copy_uchar_to_uint)
@@ -312,36 +340,46 @@ DEFUN_2(uchar_to_ulong,unsigned char,unsigned long,copy_uchar_to_ulong)
 # define uchar_to_long		uchar_to_int
 # define uchar_to_ulong		uchar_to_uint
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_2(uchar_to_float,unsigned char,float,copy_uchar_to_float)
 DEFUN_2(uchar_to_double,unsigned char,double,copy_uchar_to_double)
+#endif
 
 #if SIZEOF_INT != SIZEOF_SHORT
 DEFUN_2(short_to_int,short,int,copy_short_to_int)
 DEFUN_2(short_to_uint,short,unsigned int,copy_short_to_uint)
 DEFUN_2(short_to_long,short,long,copy_short_to_long)
 DEFUN_2(short_to_ulong,short,unsigned long,copy_short_to_ulong)
+#if SLANG_HAS_FLOAT
 DEFUN_2(short_to_float,short,float,copy_short_to_float)
 DEFUN_2(short_to_double,short,double,copy_short_to_double)
+#endif
 DEFUN_2(ushort_to_int,unsigned short,int,copy_ushort_to_int)
 DEFUN_2(ushort_to_uint,unsigned short,unsigned int,copy_ushort_to_uint)
 DEFUN_2(ushort_to_long,unsigned short,long,copy_ushort_to_long)
 DEFUN_2(ushort_to_ulong,unsigned short,unsigned long,copy_ushort_to_ulong)
+#if SLANG_HAS_FLOAT
 DEFUN_2(ushort_to_float,unsigned short,float,copy_ushort_to_float)
 DEFUN_2(ushort_to_double,unsigned short,double,copy_ushort_to_double)
+#endif
 #else
 # define short_to_int		NULL
 # define short_to_uint		NULL
 # define short_to_long		int_to_long
 # define short_to_ulong		int_to_ulong
+#if SLANG_HAS_FLOAT
 # define short_to_float		int_to_float
 # define short_to_double	int_to_double
+#endif
 # define ushort_to_int		NULL
 # define ushort_to_uint		NULL
 # define ushort_to_long		uint_to_long
 # define ushort_to_ulong	uint_to_ulong
+#if SLANG_HAS_FLOAT
 # define ushort_to_float	uint_to_float
 # define ushort_to_double	uint_to_double
 #endif
+#endif
 
 #if SIZEOF_INT != SIZEOF_LONG
 DEFUN_2(int_to_long,int,long,copy_int_to_long)
@@ -350,8 +388,10 @@ DEFUN_2(int_to_ulong,int,unsigned long,copy_int_to_ulong)
 # define int_to_long		NULL
 # define int_to_ulong		NULL
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_2(int_to_float,int,float,copy_int_to_float)
 DEFUN_2(int_to_double,int,double,copy_int_to_double)
+#endif
 
 #if SIZEOF_INT != SIZEOF_LONG
 DEFUN_2(uint_to_long,unsigned int,long,copy_uint_to_long)
@@ -360,6 +400,7 @@ DEFUN_2(uint_to_ulong,unsigned int,unsigned long,copy_uint_to_ulong)
 # define uint_to_long		NULL
 # define uint_to_ulong		NULL
 #endif
+#if SLANG_HAS_FLOAT
 DEFUN_2(uint_to_float,unsigned int,float,copy_uint_to_float)
 DEFUN_2(uint_to_double,unsigned int,double,copy_uint_to_double)
 
@@ -446,7 +487,7 @@ SLarith_get_to_double_fun (unsigned char type, unsigned int *sizeof_type)
    if (sizeof_type != NULL) *sizeof_type = da;
    return to_double;
 }
-
+#endif				       /* SLANG_HAS_FLOAT */
 /* Each element of the matrix determines how the row maps onto the column.
  * That is, let the matrix be B_ij.  Where the i,j indices refer to
  * precedence of the type.  Then,
@@ -476,8 +517,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_char_to_uint, char_to_uint},
 	{(FVOID_STAR) copy_char_to_long, char_to_long},
 	{(FVOID_STAR) copy_char_to_ulong, char_to_ulong},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_char_to_float, char_to_float},
 	{(FVOID_STAR) copy_char_to_double, char_to_double},
+#endif
      },
 
      {
@@ -489,8 +532,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_uchar_to_uint, uchar_to_uint},
 	{(FVOID_STAR) copy_uchar_to_long, uchar_to_long},
 	{(FVOID_STAR) copy_uchar_to_ulong, uchar_to_ulong},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_uchar_to_float, uchar_to_float},
 	{(FVOID_STAR) copy_uchar_to_double, uchar_to_double},
+#endif
      },
 
      {
@@ -502,8 +547,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_short_to_uint, short_to_uint},
 	{(FVOID_STAR) copy_short_to_long, short_to_long},
 	{(FVOID_STAR) copy_short_to_ulong, short_to_ulong},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_short_to_float, short_to_float},
 	{(FVOID_STAR) copy_short_to_double, short_to_double},
+#endif
      },
 
      {
@@ -515,8 +562,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_ushort_to_uint, ushort_to_uint},
 	{(FVOID_STAR) copy_ushort_to_long, ushort_to_long},
 	{(FVOID_STAR) copy_ushort_to_ulong, ushort_to_ulong},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_ushort_to_float, ushort_to_float},
 	{(FVOID_STAR) copy_ushort_to_double, ushort_to_double},
+#endif
      },
 
      {
@@ -528,8 +577,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_int_to_int, NULL},
 	{(FVOID_STAR) copy_int_to_long, int_to_long},
 	{(FVOID_STAR) copy_int_to_ulong, int_to_ulong},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_int_to_float, int_to_float},
 	{(FVOID_STAR) copy_int_to_double, int_to_double},
+#endif
      },
 
      {
@@ -541,8 +592,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_int_to_int, NULL},
 	{(FVOID_STAR) copy_uint_to_long, uint_to_long},
 	{(FVOID_STAR) copy_uint_to_ulong, uint_to_ulong},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_uint_to_float, uint_to_float},
 	{(FVOID_STAR) copy_uint_to_double, uint_to_double},
+#endif
      },
 
      {
@@ -554,8 +607,10 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_long_to_uint, NULL},
 	{(FVOID_STAR) copy_long_to_long, NULL},
 	{(FVOID_STAR) copy_long_to_long, NULL},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_long_to_float, long_to_float},
 	{(FVOID_STAR) copy_long_to_double, long_to_double},
+#endif
      },
 
      {
@@ -567,10 +622,13 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_ulong_to_uint, NULL},
 	{(FVOID_STAR) copy_long_to_long, NULL},
 	{(FVOID_STAR) copy_long_to_long, NULL},
+#if SLANG_HAS_FLOAT
 	{(FVOID_STAR) copy_ulong_to_float, ulong_to_float},
 	{(FVOID_STAR) copy_ulong_to_double, ulong_to_double},
+#endif
      },
 
+#if SLANG_HAS_FLOAT
      {
 	{(FVOID_STAR) copy_float_to_char, NULL},
 	{(FVOID_STAR) copy_float_to_uchar, NULL},
@@ -596,6 +654,7 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 	{(FVOID_STAR) copy_double_to_float, NULL},
 	{(FVOID_STAR) copy_double_to_double, NULL},
      }
+#endif
 };
 
 #define GENERIC_BINARY_FUNCTION int_int_bin_op
@@ -675,6 +734,7 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 #define ulong_cmp_function	uint_cmp_function
 #endif				       /* SIZEOF_INT != SIZEOF_LONG */
 
+#if SLANG_HAS_FLOAT
 #define GENERIC_BINARY_FUNCTION float_float_bin_op
 #define GENERIC_TYPE float
 #define POW_FUNCTION(a,b) (float)pow((double)(a),(double)(b))
@@ -706,6 +766,7 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
 #define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
 #define CMP_FUNCTION double_cmp_function
 #include "slarith.inc"
+#endif				       /* SLANG_HAS_FLOAT */
 
 #define GENERIC_UNARY_FUNCTION char_unary_op
 #define GENERIC_BIT_OPERATIONS
@@ -745,6 +806,7 @@ static Binary_Matrix_Type Binary_Matrix [MAX_ARITHMETIC_TYPES][MAX_ARITHMETIC_TY
  * not well thought out.  So, I need to use the following table.
  */
 #define MAXIMUM_ARITH_TYPE_VALUE	SLANG_FLOAT_TYPE
+
 #define IS_INTEGER_TYPE(x) \
   (((x) <= MAXIMUM_ARITH_TYPE_VALUE) \
       && (Type_Precedence_Table[x] < 8) && (Type_Precedence_Table[x] != -1))
@@ -759,7 +821,11 @@ static signed char Type_Precedence_Table [MAXIMUM_ARITH_TYPE_VALUE + 1] =
    -1,				       /* SLANG_UNDEFINED_TYPE */
    -1,				       /* SLANG_VOID_TYPE */
    4,				       /* SLANG_INT_TYPE */
+#if SLANG_HAS_FLOAT
    9,				       /* SLANG_DOUBLE_TYPE */
+#else
+   -1,
+#endif
    0,				       /* SLANG_CHAR_TYPE */
    -1,				       /* SLANG_INTP_TYPE */
    -1,				       /* SLANG_REF_TYPE */
@@ -772,7 +838,11 @@ static signed char Type_Precedence_Table [MAXIMUM_ARITH_TYPE_VALUE + 1] =
    6,				       /* SLANG_LONG_TYPE */
    7,				       /* SLANG_ULONG_TYPE */
    -1,				       /* SLANG_STRING_TYPE */
+#if SLANG_HAS_FLOAT
    8				       /* SLANG_FLOAT_TYPE */
+#else
+  -1
+#endif
 };
 
 int _SLarith_get_precedence (unsigned char type)
@@ -834,14 +904,14 @@ static int arith_bin_op_result (int op, unsigned char a_type, unsigned char b_ty
       case SLANG_AND:
 	*c_type = SLANG_CHAR_TYPE;
 	return 1;
-
+#if SLANG_HAS_FLOAT
       case SLANG_POW:
 	if (SLANG_FLOAT_TYPE == promote_to_common_type (a_type, b_type))
 	  *c_type = SLANG_FLOAT_TYPE;
 	else
 	  *c_type = SLANG_DOUBLE_TYPE;
 	return 1;
-
+#endif
       case SLANG_BAND:
       case SLANG_BXOR:
       case SLANG_BOR:
@@ -879,8 +949,12 @@ static Bin_Fun_Type Bin_Fun_Map [MAX_ARITHMETIC_TYPES] =
    uint_uint_bin_op,
    long_long_bin_op,
    ulong_ulong_bin_op,
+#if SLANG_HAS_FLOAT
    float_float_bin_op,
    double_double_bin_op
+#else
+     NULL, NULL
+#endif
 };
 
 static int arith_bin_op (int op,
@@ -1169,11 +1243,16 @@ int SLang_push_float (float f)
 }
 
 /* Double */
-static int double_push (unsigned char unused, VOID_STAR ptr)
+static int double_push (SLtype type, VOID_STAR ptr)
 {
-   (void) unused;
-   SLang_push_double (*(double *) ptr);
-   return 0;
+#if _SLANG_OPTIMIZE_FOR_SPEED
+   SLang_Object_Type obj;
+   obj.data_type = type;
+   obj.v.double_val = *(double *)ptr;
+   return SLang_push (&obj);
+#else
+   return SLclass_push_double_obj (type, *(double *) ptr);
+#endif
 }
 
 static int double_push_literal (unsigned char type, VOID_STAR ptr)
@@ -1489,7 +1568,7 @@ int _SLarith_register_types (void)
 	if (-1 == SLclass_add_unary_op (info->data_type, info->unary_fun, arith_unary_op_result))
 	  return -1;
 	
-	_SLarith_Is_Arith_Type [info->data_type] = 1;
+	_SLang_set_arith_type (info->data_type, 1);
      }
 
 #if SLANG_HAS_FLOAT
@@ -1507,7 +1586,7 @@ int _SLarith_register_types (void)
      return -1;
    if (-1 == SLclass_add_unary_op (SLANG_DOUBLE_TYPE, double_unary_op, arith_unary_op_result))
      return -1;
-   _SLarith_Is_Arith_Type [SLANG_DOUBLE_TYPE] = 2;
+   _SLang_set_arith_type (SLANG_DOUBLE_TYPE, 2);
 
    if (NULL == (cl = SLclass_allocate_class ("Float_Type")))
      return -1;
@@ -1521,7 +1600,7 @@ int _SLarith_register_types (void)
      return -1;
    if (-1 == SLclass_add_unary_op (SLANG_FLOAT_TYPE, float_unary_op, arith_unary_op_result))
      return -1;
-   _SLarith_Is_Arith_Type [SLANG_FLOAT_TYPE] = 2;
+   _SLang_set_arith_type (SLANG_FLOAT_TYPE, 2);
 #endif
 
    if (-1 == create_synonyms ())
@@ -1605,6 +1684,7 @@ int _SLarith_bin_op (SLang_Object_Type *oa, SLang_Object_Type *ob, int op)
 	SLang_Object_Type obj_a, obj_b;
 	
 	/* Handle common cases */
+#if SLANG_HAS_FLOAT
 	if ((a_type == SLANG_INT_TYPE)
 	    && (b_type == SLANG_DOUBLE_TYPE))
 	  return double_double_scalar_bin_op (oa->v.int_val, ob->v.double_val, op);
@@ -1612,7 +1692,7 @@ int _SLarith_bin_op (SLang_Object_Type *oa, SLang_Object_Type *ob, int op)
 	if ((a_type == SLANG_DOUBLE_TYPE)
 	    && (b_type == SLANG_INT_TYPE))
 	  return double_double_scalar_bin_op (oa->v.double_val, ob->v.int_val, op);
-
+#endif
 	/* Otherwise do it the hard way */
 	promote_objs (oa, ob, &obj_a, &obj_b);
 	oa = &obj_a;
@@ -1659,10 +1739,12 @@ int _SLarith_bin_op (SLang_Object_Type *oa, SLang_Object_Type *ob, int op)
       case SLANG_ULONG_TYPE:
 	return ulong_ulong_scalar_bin_op (oa->v.ulong_val, ob->v.ulong_val, op);
 #endif
+#if SLANG_HAS_FLOAT
       case SLANG_FLOAT_TYPE:
 	return float_float_scalar_bin_op (oa->v.float_val, ob->v.float_val, op);
       case SLANG_DOUBLE_TYPE:
 	return double_double_scalar_bin_op (oa->v.double_val, ob->v.double_val, op);
+#endif
      }
    
    return 1;
