@@ -173,7 +173,7 @@ static int new_object_element (SLang_Array_Type *at,
    return (*at->cl->cl_init_array_object) (at->data_type, data);
 }
 
-static int next_index (int *dims, int *max_dims, unsigned int num_dims)
+int _SLarray_next_index (int *dims, int *max_dims, unsigned int num_dims)
 {
    while (num_dims)
      {
@@ -216,7 +216,7 @@ static int do_method_for_all_elements (SLang_Array_Type *at,
 	if (-1 == (*method) (at, dims, client_data))
 	  return -1;
      }
-   while (0 == next_index (dims, max_dims, num_dims));
+   while (0 == _SLarray_next_index (dims, max_dims, num_dims));
 
    return 0;
 }
@@ -535,7 +535,8 @@ free_index_objects (SLang_Object_Type *index_objs, unsigned int num_indices)
 }
 
 static int
-pop_indices (SLang_Object_Type *index_objs, unsigned int num_indices,
+pop_indices (SLang_Array_Type *at_to_index,
+	     SLang_Object_Type *index_objs, unsigned int num_indices,
 	     int *is_index_array)
 {
    unsigned int i;
@@ -566,9 +567,16 @@ pop_indices (SLang_Object_Type *index_objs, unsigned int num_indices,
 
 	     if (at->num_dims == 1)
 	       {
-		  if ((num_indices == 1)
-		      && (0 == (at->flags & SLARR_DATA_VALUE_IS_RANGE)))
-		    *is_index_array = 1;
+		  /* Note: Suppose a = Int_Type[10, 10].  Then the check
+		   * on the range flag prevents this from being indexed by
+		   * a range such as 'a[[0:99]]'
+		   */
+		  if (num_indices == 1)
+		    {
+		       if ((at_to_index->num_dims > 1)
+			   || (0 == (at->flags & SLARR_DATA_VALUE_IS_RANGE)))
+			 *is_index_array = 1;
+		    }
 	       }
 	     else
 	       {
@@ -976,7 +984,7 @@ aget_from_indices (SLang_Array_Type *at,
 	  }
 	new_data += sizeof_type;
      }
-   while (0 == next_index (map_indices, max_dims, num_indices));
+   while (0 == _SLarray_next_index (map_indices, max_dims, num_indices));
 
    if (new_at != NULL)
      {
@@ -1078,14 +1086,42 @@ static int aget_from_array (unsigned int num_indices)
    if (-1 == pop_array (&at, 1))
      return -1;
 
-   if (-1 == pop_indices (index_objs, num_indices, &is_index_array))
+   if (-1 == pop_indices (at, index_objs, num_indices, &is_index_array))
      {
 	SLang_free_array (at);
 	return -1;
      }
 
    if (is_index_array == 0)
-     ret = aget_from_indices (at, index_objs, num_indices);
+     {
+#if _SLANG_OPTIMIZE_FOR_SPEED
+	if ((num_indices == 1) && (index_objs[0].data_type == SLANG_INT_TYPE)
+	    && (0 == (at->flags & (SLARR_DATA_VALUE_IS_RANGE|SLARR_DATA_VALUE_IS_POINTER)))
+	    && (1 == at->num_dims)
+	    && (at->data != NULL))
+	  {
+	     int ofs = index_objs[0].v.int_val;
+	     if (ofs < 0) ofs += at->dims[0];
+	     if ((ofs >= at->dims[0]) || (ofs < 0))
+	       ret = aget_from_indices (at, index_objs, num_indices);
+	     else switch (at->data_type)
+	       {
+		case SLANG_INT_TYPE:
+		  ret = SLclass_push_int_obj (SLANG_INT_TYPE, *((int *)at->data + ofs));
+		  break;
+#if SLANG_HAS_FLOAT
+		case SLANG_DOUBLE_TYPE:
+		  ret = SLclass_push_double_obj (SLANG_DOUBLE_TYPE, *((double *)at->data + ofs));
+		  break;
+#endif
+		default:
+		  ret = aget_from_indices (at, index_objs, num_indices);
+	       }
+	  }
+	else
+#endif
+	ret = aget_from_indices (at, index_objs, num_indices);
+     }
    else
      ret = aget_from_index_array (at, index_objs[0].v.array_val);
 
@@ -1206,6 +1242,9 @@ int _SLarray_aget (void)
       case SLANG_ARRAY_TYPE:
 	break;
 
+      case SLANG_ASSOC_TYPE:
+	return _SLassoc_aget (type, num_indices);
+
       default:
 	aget_fun = _SLclass_get_class (type)->cl_aget;
 	if (NULL != aget_fun)
@@ -1351,7 +1390,7 @@ aput_from_indices (SLang_Array_Type *at,
 
 	data_to_put += data_increment;
      }
-   while (0 == next_index (map_indices, max_dims, num_indices));
+   while (0 == _SLarray_next_index (map_indices, max_dims, num_indices));
 
    ret = 0;
 
@@ -1462,6 +1501,9 @@ int _SLarray_aput (void)
       case SLANG_ARRAY_TYPE:
 	break;
 
+      case SLANG_ASSOC_TYPE:
+	return _SLassoc_aput (type, num_indices);
+
       default:
 	if (NULL != (aput_fun = _SLclass_get_class (type)->cl_aput))
 	  return (*aput_fun) (type, num_indices);
@@ -1479,14 +1521,42 @@ int _SLarray_aput (void)
 	return -1;
      }
 
-   if (-1 == pop_indices (index_objs, num_indices, &is_index_array))
+   if (-1 == pop_indices (at, index_objs, num_indices, &is_index_array))
      {
 	SLang_free_array (at);
 	return -1;
      }
 
    if (is_index_array == 0)
-     ret = aput_from_indices (at, index_objs, num_indices);
+     {
+#if _SLANG_OPTIMIZE_FOR_SPEED
+	if ((num_indices == 1) && (index_objs[0].data_type == SLANG_INT_TYPE)
+	    && (0 == (at->flags & (SLARR_DATA_VALUE_IS_RANGE|SLARR_DATA_VALUE_IS_POINTER)))
+	    && (1 == at->num_dims)
+	    && (at->data != NULL))
+	  {
+	     int ofs = index_objs[0].v.int_val;
+	     if (ofs < 0) ofs += at->dims[0];
+	     if ((ofs >= at->dims[0]) || (ofs < 0))
+	       ret = aput_from_indices (at, index_objs, num_indices);
+	     else switch (at->data_type)
+	       {
+		case SLANG_INT_TYPE:
+		  ret = SLang_pop_integer (((int *)at->data + ofs));
+		  break;
+#if SLANG_HAS_FLOAT
+		case SLANG_DOUBLE_TYPE:
+		  ret = SLang_pop_double ((double *)at->data + ofs, NULL, NULL);
+		  break;
+#endif
+		default:
+		  ret = aput_from_indices (at, index_objs, num_indices);
+	       }
+	  }
+	else
+#endif
+	  ret = aput_from_indices (at, index_objs, num_indices);
+     }
    else
      ret = aput_from_index_array (at, index_objs[0].v.array_val);
 
@@ -1509,6 +1579,33 @@ static int push_element_at_index (SLang_Array_Type *at, int indx)
 static SLang_Name_Type *Sort_Function;
 static SLang_Array_Type *Sort_Array;
 
+#if _SLANG_OPTIMIZE_FOR_SPEED
+static int double_sort_fun (int *a, int *b)
+{
+   double *da, *db;
+   
+   da = (double *) Sort_Array->data;
+   db = da + *b;
+   da = da + *a;
+
+   if (*da > *db) return 1;
+   if (*da == *db) return 0;
+   return -1;
+}
+static int int_sort_fun (int *a, int *b)
+{
+   int *da, *db;
+   
+   da = (int *) Sort_Array->data;
+   db = da + *b;
+   da = da + *a;
+
+   if (*da > *db) return 1;
+   if (*da == *db) return 0;
+   return -1;
+}
+#endif
+
 static int sort_cmp_fun (int *a, int *b)
 {
    int cmp;
@@ -1626,9 +1723,18 @@ static void sort_array (void)
      }
    else
      {
-	sort_fun = builtin_sort_cmp_fun;
 	if (-1 == SLang_pop_array (&at, 1))
 	  return;
+	
+#if _SLANG_OPTIMIZE_FOR_SPEED
+	if (at->data_type == SLANG_DOUBLE_TYPE)
+	  sort_fun = double_sort_fun;
+	else if (at->data_type == SLANG_INT_TYPE)
+	  sort_fun = int_sort_fun;
+	else
+#endif
+	  sort_fun = builtin_sort_cmp_fun;
+
 	if (at->cl->cl_cmp == NULL)
 	  {
 	     SLang_verror (SL_NOT_IMPLEMENTED, 
@@ -1878,13 +1984,13 @@ int _SLarray_inline_implicit_array (void)
    int int_vals[3];
 #if SLANG_HAS_FLOAT
    double double_vals[3];
+   int is_int;
 #endif
    int has_vals[3];
    unsigned int i, count;
    SLang_Array_Type *at;
    int precedence;
    unsigned char type;
-   int is_int;
 
    count = SLang_Num_Function_Args;
 
@@ -1938,7 +2044,9 @@ int _SLarray_inline_implicit_array (void)
 	   default:
 	     if (-1 == SLang_pop_integer (int_vals + i))
 	       return -1;
+#if SLANG_HAS_FLOAT
 	     double_vals[i] = (double) int_vals[i];
+#endif
 	  }
      }
 
@@ -2090,17 +2198,18 @@ static SLang_Array_Type *concat_arrays (unsigned int count)
 
 int _SLarray_inline_array (void)
 {
-   SLang_Object_Type *obj;
+   SLang_Object_Type *obj, *objmin;
    unsigned char type, this_type;
    unsigned int count;
    SLang_Array_Type *at;
 
-   obj = _SLStack_Pointer;
+   obj = _SLang_get_run_stack_pointer ();
+   objmin = _SLang_get_run_stack_base ();
 
    count = SLang_Num_Function_Args;
    type = 0;
 
-   while ((count > 0) && (--obj >= _SLRun_Stack))
+   while ((count > 0) && (--obj >= objmin))
      {
 	this_type = obj->data_type;
 
@@ -2118,22 +2227,22 @@ int _SLarray_inline_array (void)
 	   case SLANG_ARRAY_TYPE:
 	     type = SLANG_ARRAY_TYPE;
 	     break;
-
+#if SLANG_HAS_COMPLEX
 	   case SLANG_COMPLEX_TYPE:
-	     if (0 == _SLarith_Is_Arith_Type[type])
+	     if (0 == _SLang_is_arith_type (type))
 	       goto type_mismatch;
 	     
 	     type = this_type;
 	     break;
-	     
+#endif
 	   default:
-	     if (0 == _SLarith_Is_Arith_Type[this_type])
+	     if (0 == _SLang_is_arith_type(this_type))
 	       goto type_mismatch;
 	     
 	     if (type == SLANG_COMPLEX_TYPE)
 	       break;
 	     
-	     if (0 == _SLarith_Is_Arith_Type[type])
+	     if (0 == _SLang_is_arith_type (type))
 	       goto type_mismatch;
 	     
 	     if (_SLarith_get_precedence (this_type) > _SLarith_get_precedence (type))
