@@ -11,10 +11,6 @@
 #include "slang.h"
 #include "_slang.h"
 
-#if _SLANG_OPTIMIZE_FOR_SPEED
-unsigned char _SLclass_Class_Type [256];
-#endif
-
 static SLang_Class_Type *Registered_Types[256];
 SLang_Class_Type *_SLclass_get_class (unsigned char type)
 {
@@ -532,7 +528,7 @@ int _SLclass_copy_class (unsigned char to, unsigned char from)
 
 #if _SLANG_OPTIMIZE_FOR_SPEED
    if (to != SLANG_UNDEFINED_TYPE)
-     _SLclass_Class_Type [to] = cl->cl_class_type;
+     _SLang_set_class_type(to, cl->cl_class_type);
 #endif
    return 0;
 }
@@ -622,7 +618,7 @@ int SLclass_register_class (SLang_Class_Type *cl, unsigned char type, unsigned i
 
 #if _SLANG_OPTIMIZE_FOR_SPEED
    if (type != SLANG_UNDEFINED_TYPE)
-     _SLclass_Class_Type [type] = class_type;
+     _SLang_set_class_type (type, class_type);
 #endif
 
    if (type_size == 0)
@@ -907,7 +903,7 @@ static char *get_binary_op_string (int op)
      };
 
    if ((op > SLANG_MOD) || (op <= 0))
-     return "-??-";
+     return "- ?? -";		       /* Note: -??- is a trigraph (sigh) */
    return ops[op - 1];
 }
 
@@ -1289,13 +1285,6 @@ void SLang_free_value (unsigned char type, VOID_STAR v)
  * pointer versions are in slang.c
  */
 #if SLANG_HAS_FLOAT
-int SLclass_push_double_obj (unsigned char type, double x)
-{
-   SLang_Object_Type obj;
-   obj.data_type = type;
-   obj.v.double_val = x;
-   return SLang_push (&obj);
-}
 int SLclass_push_float_obj (unsigned char type, float x)
 {
    SLang_Object_Type obj;
@@ -1303,7 +1292,6 @@ int SLclass_push_float_obj (unsigned char type, float x)
    obj.v.float_val = x;
    return SLang_push (&obj);
 }
-
 #endif
 
 int SLclass_push_long_obj (unsigned char type, long x)
