@@ -133,7 +133,8 @@ static int import_from_library (char *name,
 	/* Purify reports that dlerror returns a pointer that generates UMR
 	 * errors.  There is nothing that I can do about that....
 	 */
-	if (NULL == strchr (file, '/'))
+	if ((NULL == strchr (file, '/'))
+	    && (strlen(file) < sizeof(filebuf)))
 	  {
 	     err = (char *) dlerror ();
 	     if (err != NULL)
@@ -202,10 +203,11 @@ static int import_from_library (char *name,
 
 static void import_module (void)
 {
-   char module_name[256];
-   char symbol_name[256];
-   char deinit_name[256];
-   char ns_init_name[256];
+#define MAX_MODULE_NAME_SIZE 256
+   char module_name[MAX_MODULE_NAME_SIZE];
+   char symbol_name[MAX_MODULE_NAME_SIZE];
+   char deinit_name[MAX_MODULE_NAME_SIZE];
+   char ns_init_name[MAX_MODULE_NAME_SIZE];
    char *path;
    char *file;
    char *module;
@@ -223,6 +225,14 @@ static void import_module (void)
 	return;
      }
 
+   if (strlen (module) >= MAX_MODULE_NAME_SIZE)
+     {
+	SLang_verror (SL_INVALID_PARM, "module name too long");
+	SLang_free_slstring (module);
+	SLang_free_slstring (ns);
+	return;
+     }
+   
    _SLsnprintf (symbol_name, sizeof(symbol_name), "init_%s_module", module);
    _SLsnprintf (module_name, sizeof(module_name), "%s-module.so", module);
    _SLsnprintf (deinit_name, sizeof(deinit_name), "deinit_%s_module", module);
@@ -233,7 +243,7 @@ static void import_module (void)
    else file = NULL;
 
    if ((file == NULL)
-       && (NULL != (path = getenv (MODULE_PATH_ENV_NAME))))
+       && (NULL != (path = _SLsecure_getenv (MODULE_PATH_ENV_NAME))))
      file = SLpath_find_file_in_path (path, module_name);
 
    if (file == NULL)
@@ -261,7 +271,7 @@ static char *get_import_module_path (void)
    char *path;
    if (Module_Path != NULL)
      return Module_Path;
-   if (NULL != (path = getenv (MODULE_PATH_ENV_NAME)))
+   if (NULL != (path = _SLsecure_getenv (MODULE_PATH_ENV_NAME)))
      return path;
    return MODULE_INSTALL_DIR;
 }
