@@ -12,7 +12,9 @@
 int SLang_init_all (void)
 {
    if ((-1 == SLang_init_slang ())
+#if SLANG_HAS_FLOAT
        || (-1 == SLang_init_slmath ())
+#endif
        || (-1 == SLang_init_posix_dir ())
        || (-1 == SLang_init_posix_process ())
        || (-1 == SLang_init_stdio ())
