@@ -10,6 +10,7 @@
 
 #include <math.h>
 
+#if SLANG_HAS_FLOAT
 #include "slang.h"
 #include "_slang.h"
 
@@ -566,4 +567,4 @@ int SLang_init_slmath (void)
 
    return 0;
 }
-
+#endif				       /* SLANG_HAS_FLOAT */
