@@ -4,6 +4,7 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
+#define _GNU_SOURCE
 #include "slinclud.h"
 
 #include <ctype.h>
@@ -47,10 +48,11 @@ char *SLmake_nstring (char *str, unsigned int n)
 
 void SLmake_lut (unsigned char *lut, unsigned char *range, unsigned char reverse)
 {
-   register unsigned char *l = lut, *lmax = lut + 256;
+   /* register unsigned char *l = lut, *lmax = lut + 256; */
    int i, r1, r2;
 
-   while (l < lmax) *l++ = reverse;
+   memset ((char *)lut, reverse, 256);
+   /* while (l < lmax) *l++ = reverse; */
    reverse = !reverse;
 
    r1 = *range++;
@@ -562,3 +564,42 @@ int SLatoi (unsigned char *s)
 {
    return (int) SLatol (s);
 }
+
+#if !defined(HAVE_ISSETUGID) && defined(__GLIBC__) && (__GLIBC__ >= 2)
+extern int __libc_enable_secure;
+# define HAVE___LIBC_ENABLE_SECURE 1
+#endif
+
+int _SLsecure_issetugid (void)
+{
+#ifdef HAVE_ISSETUGID
+   return (1 == issetugid ());
+#else
+# ifdef HAVE___LIBC_ENABLE_SECURE
+   return __libc_enable_secure;
+# else
+#  if defined(HAVE_GETUID) && defined(HAVE_GETEUID) && defined(HAVE_GETGID) && defined(HAVE_GETEUID)
+   static int enable_secure;
+   if (enable_secure == 0)
+    {
+       if ((getuid () != geteuid ()) 
+	   || (getgid () != getegid ()))
+	 enable_secure = 1;
+       else
+	 enable_secure = -1;
+    }
+   return (enable_secure == 1);
+#  else
+   return 0;
+#  endif
+# endif
+#endif
+}
+
+/* Like getenv, except if running as setuid or setgid, returns NULL */
+char *_SLsecure_getenv (char *s)
+{
+   if (_SLsecure_issetugid ())
+     return NULL;
+   return getenv (s);
+}
