@@ -98,35 +98,37 @@
 #include "slang.h"
 #include "_slang.h"
 
+typedef struct
+{
+   struct stat st;
+   int st_opt_attrs;
+}
+Stat_Type;
+
+static SLang_CStruct_Field_Type Stat_Struct [] =
+{
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_dev, "st_dev", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_ino, "st_ino", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_mode, "st_mode", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_nlink, "st_nlink", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_uid, "st_uid", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_gid, "st_gid", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_rdev, "st_rdev", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_size, "st_size", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_atime, "st_atime", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_mtime, "st_mtime", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_ctime, "st_ctime", 0),
+   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st_opt_attrs, "st_opt_attrs", 0),
+   SLANG_END_CSTRUCT_TABLE
+};
+
 static int push_stat_struct (struct stat *st, int opt_attrs)
 {
-   char *field_names [12];
-   unsigned char field_types[12];
-   VOID_STAR field_values [12];
-   int int_values [12];
-   unsigned int i;
-
-   field_names [0] = "st_dev"; int_values [0] = (int) st->st_dev;
-   field_names [1] = "st_ino"; int_values [1] = (int) st->st_ino;
-   field_names [2] = "st_mode"; int_values [2] = (int) st->st_mode;
-   field_names [3] = "st_nlink"; int_values [3] = (int) st->st_nlink;
-   field_names [4] = "st_uid"; int_values [4] = (int) st->st_uid;
-   field_names [5] = "st_gid"; int_values [5] = (int) st->st_gid;
-   field_names [6] = "st_rdev"; int_values [6] = (int) st->st_rdev;
-   field_names [7] = "st_size"; int_values [7] = (int) st->st_size;
-   field_names [8] = "st_atime"; int_values [8] = (int) st->st_atime;
-   field_names [9] = "st_mtime"; int_values [9] = (int) st->st_mtime;
-   field_names [10] = "st_ctime"; int_values [10] = (int) st->st_ctime;
-
-   field_names [11] = "st_opt_attrs"; int_values[11] = opt_attrs;
-
-   for (i = 0; i < 12; i++)
-     {
-	field_types [i] = SLANG_INT_TYPE;
-	field_values [i] = (VOID_STAR) (int_values + i);
-     }
-
-   return SLstruct_create_struct (12, field_names, field_types, field_values);
+   Stat_Type s;
+   
+   s.st = *st;
+   s.st_opt_attrs = opt_attrs;
+   return SLang_push_cstruct ((VOID_STAR) &s, Stat_Struct);
 }
 
 static void stat_cmd (char *file)
