@@ -1,4 +1,4 @@
-/* Copyright (c) 1996, 1999, 2001 John E. Davis
+/* Copyright (c) 1996, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -22,16 +22,21 @@
  *
  * #iftrue
  * #ifnfalse
+ * #if true
+ * #if !false
  *	- always True
  *
  * #iffalse
  * #ifntrue
+ * #if false
+ * #if !true
  *	- always False
  *
  * #if$ENV
  *	- True if the enviroment variable ENV is set
  *
  * #ifn$ENV
+ * #if !$ENV
  *	- True if the enviroment variable ENV is not set
  *
  * #if$ENV TOKEN1 TOKEN2 ...
@@ -39,6 +44,7 @@
  *	  any of TOKEN1 TOKEN2 ...
  *
  * #ifn$ENV TOKEN1 TOKEN2 ...
+ * #if !$ENV TOKEN1 TOKEN2 ...
  *	- True if the contents of enviroment variable ENV do not match
  *	  any of TOKEN1 TOKEN2 ...
  *
@@ -46,10 +52,61 @@
  *		'?' - match any single character
  *		'*' - match any number of characters
  *
+ * #ifexists TOKEN
+ * #ifnexists TOKEN
+ * #if !exists TOKEN
+ *	- check if a variable/function exists
+ *
+ * #ifeval EXPRESSION
+ * #ifneval EXPRESSION
+ * #if !eval TOKEN
+ *	- evaluates the EXPRESSION as an SLang expression
+ *
+ * #if (EXPRESSION)
+ * #if !(EXPRESSION)
+ *	- as per '#ifeval' / '#ifneval',
+ *	  evaluates the EXPRESSION as a SLang expression
+ *	- using '#ifn (EXPRESSION)' is possible, but deprecated
+ *
  * #elif...
  * #else
  * #endif
  *
+ * #stop
+ *	- stop reading the entire file here, provided that the line
+ *	  would have been executed
+ *	eg:
+ *		#iffalse
+ *		#  stop
+ *		#endif
+ *	would NEVER stop
+ *
+ * #<TAG>
+ *	- start embedded text region
+ * #</TAG>
+ *	- end embedded text region
+ *
+ *	All text, include other preprocessing directives, that occurs between
+ *	the '#<TAG>' and '#</TAG>' directives will be ignored.
+ *	This is useful for embedding other code or documentation.
+ *	eg:
+ *		#<latex>
+ *		\chapter{My Documentation Effort}
+ *		#</latex>
+ *	NB: * although the current implementation only looks for sequences
+ *	      '#<' and '#</', it is advisable to use the full '<TAG>' form
+ *	      for documentation purposes and to avoid future surprises.
+ *          * do NOT attempt to nest these constructions
+ *
+ * GENERAL NOTES:
+ *   Apart from the '#ifdef' and '#ifndef' constructions, we are quite
+ *   generous with accepting whitespace and the alternative '!' form.
+ *   eg.,
+ *	#ifTEST
+ *	#ifnTEST
+ *	#if TEST
+ *	#if !TEST
+ *	#if ! TEST
  *
  * mj olesen
  *----------------------------------------------------------------------*/
@@ -61,8 +118,8 @@
 #include "_slang.h"
 /*}}}*/
 
-int (*SLprep_exists_hook) (char *, char);
-int (*_SLprep_eval_hook) (char *);
+int (*SLprep_exists_hook) (char *, char);	/* in "slang.h" */
+int (*_SLprep_eval_hook) (char *);		/* in "_slang.h" */
 
 /*{{{ SLprep_open_prep (), SLprep_close_prep () */
 int SLprep_open_prep (SLPreprocess_Type *pt)
@@ -271,36 +328,75 @@ static int is_env_defined (char *buf, char comment)	/*{{{*/
 int SLprep_line_ok (char *buf, SLPreprocess_Type *pt)	/*{{{*/
 {
    int level, prev_exec_level, exec_level;
+   unsigned int flags;
 
    if ((buf == NULL) || (pt == NULL)) return 1;
 
-   if (*buf != pt->preprocess_char)
-     {
-	if (pt->this_level != pt->exec_level)
-	  return 0;
-
-	if (*buf == '\n') return pt->flags & SLPREP_BLANK_LINES_OK;
-	if (*buf == pt->comment_char) return pt->flags & SLPREP_COMMENT_LINES_OK;
-
-	return 1;
-     }
-
-   level = pt->this_level;
-   exec_level = pt->exec_level;
-   prev_exec_level = pt->prev_exec_level;
-
-   buf++;
-
-   /* Allow '#!' to pass.  This could be a shell script with something
-    like '#! /local/bin/slang'  */
-   if ((*buf == '!') && (pt->preprocess_char == '#'))
+   /* the '#stop' marker was already reached */
+   if (pt->flags & SLPREP_STOP_READING)
      return 0;
 
-   /* Allow whitespace as in '#   ifdef'  */
-   while ((*buf == ' ') || (*buf == '\t')) buf++;
-   if (*buf < 'a') return (level == exec_level);
+   /* local bookkeeping */
+   level           = pt->this_level;
+   exec_level      = pt->exec_level;
+   prev_exec_level = pt->prev_exec_level;
+   flags = pt->flags;
 
-   if (!strncmp(buf, "endif", 5))
+   if (*buf != pt->preprocess_char)
+     {
+	/* ignore out-of-context or embedded text */
+	if ((level != exec_level) || (flags & SLPREP_EMBEDDED_TEXT))
+	  return 0;
+
+	if (*buf == '\n') return flags & SLPREP_BLANK_LINES_OK;
+	if (*buf == pt->comment_char) return flags & SLPREP_COMMENT_LINES_OK;
+	
+	return 1;
+     }
+   
+   buf++;
+   
+   /*
+    * Always allow '#!' to pass. This could be a shell script
+    * with something like '#! /usr/local/bin/slang'
+    */
+   if ((*buf == '!') && (pt->preprocess_char == '#'))
+     return 0;
+   
+   /* Allow whitespace as in '#  ifdef'  */
+   while ((*buf == ' ') || (*buf == '\t')) buf++;
+   
+   /*
+    * quick and dirty coding for '#<TAG>' and '#</TAG>'
+    * only bothers to differentiate between '#<' and '#</'
+    * never attempt to nest these constructions!!
+    */
+   if (*buf == '<')
+     {
+	buf++;
+	if (*buf == '/') 	/* likely a '#</TAG>' */
+	  pt->flags &= ~SLPREP_EMBEDDED_TEXT;	
+	else			/* likely a '#<TAG>' */
+	  pt->flags |= SLPREP_EMBEDDED_TEXT;
+	
+	return 0;	    
+     }
+   
+   if (pt->flags & SLPREP_EMBEDDED_TEXT)
+     return 0;		/* embedded text - ignore everything */
+   
+   
+   if ((*buf < 'a') || (*buf > 'z'))	/* something weird */
+     return (level == exec_level);
+   
+   if ( !strncmp(buf, "stop", 4) )
+     {
+	if (level == exec_level)	/* signal stop if we're in scope */
+	  pt->flags |= SLPREP_STOP_READING;
+	return 0;	/* swallow this tag */
+     }
+   
+   if (!strncmp(buf, "endif", 5)) 
      {
 	if (level == exec_level)
 	  {
@@ -311,91 +407,112 @@ int SLprep_line_ok (char *buf, SLPreprocess_Type *pt)	/*{{{*/
 	if (level < prev_exec_level) prev_exec_level = level;
 	goto done;
      }
-
-   if ((buf[0] == 'e') && (buf[1] == 'l'))   /* else, elifdef, ... */
+   
+   if ((buf[0] == 'e') && (buf[1] == 'l'))  /* else, elifdef, ... */
      {
 	if ((level == exec_level + 1)
-	    && (prev_exec_level != level))
+	    && (prev_exec_level != level)) 
 	  {
 	     /* We are in position to execute */
 	     buf += 2;
 	     if ((buf[0] == 's') && (buf[1] == 'e'))
 	       {
-		  /* "else" */
+		  /* 'else' */
 		  exec_level = level;
 		  goto done;
 	       }
-
-	     /* drop through to ifdef testing.  First set variable
-	      * to values appropriate for ifdef testing.
+	     
+	     /*
+	      * drop through to 'if' testing.
+	      * First set variable to value appropriate for 'if' testing.
 	      */
 	     level--;		       /* now == to exec level */
-	  }
+	  } 
 	else
 	  {
-	     if (level == exec_level)
+	     if (level == exec_level) 
 	       {
 		  exec_level--;
 	       }
 	     goto done;
 	  }
      }
-
-   if ((buf[0] == 'i') && (buf[1] == 'f'))
+   
+   if ((buf[0] == 'i') && (buf[1] == 'f')) 
      {
-	int truth;
-
-	if (level != exec_level)
-	  {
-	     /* Not interested */
-	     level++;
-	     goto done;
-	  }
-
-	level++;
-
+	int test  = 0;			/* fallback value */
+	int truth = 1;
 	buf += 2;
-	if (buf[0] == 'n')
+	
+	if (level != exec_level) 
 	  {
-	     truth = 0;
+	     level++;
+	     goto done;			/* Not interested */
+	  }
+	level++;
+	
+	if (buf[0] == 'n') 
+	  {
+	     truth = !truth;
 	     buf++;
 	  }
-	else truth = 1;
-
+	
+	/* for 'ifdef' and 'ifndef' we are done */
 	if (!strncmp (buf, "def", 3))
-	  truth = (truth == is_any_defined(buf + 3, pt->comment_char));
-
-	else if (!strncmp (buf, "false", 5))
-	  truth = !truth;
-
-	else if (*buf == '$')
-	  truth = (truth == is_env_defined (buf + 1, pt->comment_char));
-
-	else if (!strncmp (buf, "exists", 6)
-		 && (SLprep_exists_hook != NULL))
-	  truth = (truth == (*SLprep_exists_hook)(buf + 6, pt->comment_char));
-
-	else if (!strncmp (buf, "eval", 4)
-		 && (_SLprep_eval_hook != NULL))
-	  truth = (truth == (*_SLprep_eval_hook) (buf + 4));
-			   
-	else if (0 != strncmp (buf, "true", 4))
-	  return 1;		       /* let it bomb */
-
-	if (truth)
 	  {
-	     exec_level = level;
-	     prev_exec_level = exec_level;
+	     test = is_any_defined(buf + 3, pt->comment_char);
+	  } 
+	else 
+	  {
+	     /*
+	      * the '#ifn' construction cannot have whitespace or '!'
+	      * for the other forms, we can also accept
+	      * 'if!..' instead of 'ifn...', or even 'if ...' or 'if ! ...'
+	      */
+	     if (truth)
+	       {
+		  /* Allow some whitespace */
+		  while ((*buf == ' ') || (*buf == '\t')) buf++;
+		  
+		  if (*buf == '!') 
+		    {
+		       /* the 'if !' form */	
+		       truth = !truth;
+		       buf++;
+		       while ((*buf == ' ') || (*buf == '\t')) buf++;
+		    }
+	       }
+	     
+	     if (*buf == '$') 
+	       test = is_env_defined (buf + 1, pt->comment_char);
+	     else if (*buf == '('
+		      && (_SLprep_eval_hook != NULL)) 
+	       test = (*_SLprep_eval_hook) (buf);
+	     else if (!strncmp (buf, "eval", 4)
+		      && (_SLprep_eval_hook != NULL))
+	       test = (*_SLprep_eval_hook) (buf + 4);
+	     else if (!strncmp (buf, "exists", 6)
+		      && (SLprep_exists_hook != NULL))
+	       test = (*SLprep_exists_hook) (buf + 6, pt->comment_char);
+	     else if (!strncmp (buf, "true", 4))
+	       test = 1;
+	     else if (strncmp (buf, "false", 5))
+	       return 1;	/* unknown - let it bomb */
 	  }
+	
+	if (truth == test) prev_exec_level = exec_level = level;
+     }
+   else
+     {
+	return 1;  /* let it bomb */
      }
-   else return 1;  /* let it bomb. */
 
    done:
+   
+   if (exec_level < 0) return 1;	/* bad level - let it bomb */
 
-   if (exec_level < 0) return 1;
-
-   pt->this_level = level;
-   pt->exec_level = exec_level;
+   pt->this_level      = level;
+   pt->exec_level      = exec_level;
    pt->prev_exec_level = prev_exec_level;
    return 0;
 }
@@ -403,23 +520,48 @@ int SLprep_line_ok (char *buf, SLPreprocess_Type *pt)	/*{{{*/
 
 /*{{{ main() - for testing only */
 #if 0
-int main ()
+int main (int argc, char *argv[])
 {
+   int i;
    char buf[1024];
    SLPreprocess_Type pt;
 
    SLprep_open_prep (&pt);
-
    SLdefine_for_ifdef ("UNIX");
 
+   /* super cheap getopts */
+   for (i = 1; i < argc; i++) 
+     {
+	char *p = argv[i];
+	if (*p++ != '-') break;
+	if (*p == '-')   break;
+
+	if (*p == 'D')
+	  {
+	     SLdefine_for_ifdef (p + 1);
+	     continue;
+	  }
+	
+	while (*p) {
+	   switch (*p) {
+	    case 'B': pt.flags |= (SLPREP_BLANK_LINES_OK);	break;
+	    case 'C': pt.flags |= (SLPREP_COMMENT_LINES_OK);	break;
+	    case 'c': if (p[1]) pt.comment_char = *++p;		break;
+	    case 'p': if (p[1]) pt.preprocess_char = *++p;	break;
+	    default:
+	      fprintf (stderr, "unknown flag '%c'\n", *p);
+	      break;
+	   }
+	   p++;
+	}
+     }
+   
    while (NULL != fgets (buf, sizeof (buf) - 1, stdin))
      {
 	if (SLprep_line_ok (buf, &pt))
-	  {
-	     fputs (buf, stdout);
-	  }
+	  fputs (buf, stdout);
      }
-
+   
    SLprep_close_prep (&pt);
    return 0;
 }
