@@ -14,14 +14,6 @@
 #include "slang.h"
 #include "_slang.h"
 
-static char *skip_whitespace (char *s)
-{
-   while (isspace (*s))
-     s++;
-   
-   return s;
-}
-
 static void init_map (unsigned char map[256], int base)
 {
    memset ((char *) map, 0xFF, 256);
@@ -253,7 +245,9 @@ static int parse_double (char **sp, char *smax, double *d)
      *b = 0;
    
    *sp = s;
-#if HAVE_STRTOD
+   
+   /* fprintf (stdout, "buf='%s'\n", buf); */
+#ifdef HAVE_STRTOD
    *d = sign * strtod (buf, NULL);
 #else
    *d = sign * atof (buf);
@@ -430,7 +424,7 @@ int _SLang_sscanf (void)
 
 	if (isspace (chf))
 	  {
-	     s = skip_whitespace (s);
+	     s = _SLskip_whitespace (s);
 	     continue;
 	  }
 	
@@ -483,7 +477,7 @@ int _SLang_sscanf (void)
 	status = -1;
 
 	if ((chf != 'c') && (chf != '['))
-	  s = skip_whitespace (s);
+	  s = _SLskip_whitespace (s);
 
 	if (has_width)
 	  {
@@ -697,7 +691,7 @@ double _SLang_atof (char *s)
 {
    double x;
 
-   s = skip_whitespace (s);
+   s = _SLskip_whitespace (s);
    errno = 0;
 
    if (1 != parse_double (&s, s + strlen (s), &x))
