@@ -174,22 +174,6 @@ static void intrin_putenv (void) /*{{{*/
 
 #endif
 
-static void lang_print_stack (void) /*{{{*/
-{
-   char buf[32];
-   unsigned int n;
-
-   n = (unsigned int) (_SLStack_Pointer - _SLRun_Stack);
-   while (n)
-     {
-	n--;
-	sprintf (buf, "(%u)", n);
-	_SLdump_objects (buf, _SLRun_Stack + n, 1, 1);
-     }
-}
-
-/*}}}*/
-
 static void byte_compile_file (char *f, int *m)
 {
    SLang_byte_compile_file (f, *m);
@@ -262,9 +246,10 @@ static void intrin_int (void) /*{{{*/
 static char *
 intrin_function_name (void)
 {
-   if (NULL == _SLang_Current_Function_Name)
+   char *name;
+   if (NULL == (name = _SLang_current_function_name ()))
      return "";
-   return _SLang_Current_Function_Name;
+   return name;
 }
 
 static void intrin_message (char *s)
@@ -516,6 +501,11 @@ static void intrin_get_reference (char *name)
    _SLang_push_ref (1, (VOID_STAR) _SLlocate_name (name));
 }
 
+static void intrin_get_namespaces (void)
+{
+   SLang_push_array (_SLns_list_namespaces (), 1);
+}
+
 #ifdef HAVE_SYS_UTSNAME_H
 # include <sys/utsname.h>
 #endif
@@ -617,6 +607,10 @@ static int eqs_intrinsic (void)
    return eqs;
 }
 
+static void lang_print_stack (void)
+{
+   (void) _SLang_dump_stack ();
+}
 
 static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
 {
@@ -642,6 +636,7 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_S("integer",  intrin_integer, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("system",  SLsystem, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("_apropos",  intrin_apropos, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("_get_namespaces", intrin_get_namespaces, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("_trace_function",  _SLang_trace_fun, SLANG_VOID_TYPE),
 #if SLANG_HAS_FLOAT
    MAKE_INTRINSIC_S("atof", _SLang_atof, SLANG_DOUBLE_TYPE),
