@@ -41,6 +41,8 @@
 
 #include <errno.h>
 
+#include <ctype.h>
+
 /* #define SL_APP_WANTS_FOREACH */
 #include "slang.h"
 #include "_slang.h"
@@ -305,7 +307,7 @@ static int stdio_fclose (SL_File_Table_Type *t)
    return ret;
 }
 
-static int read_one_line (FILE *fp, char **strp, unsigned int *lenp)
+static int read_one_line (FILE *fp, char **strp, unsigned int *lenp, int trim_trailing)
 {
    char buf[512];
    char *str;
@@ -352,6 +354,21 @@ static int read_one_line (FILE *fp, char **strp, unsigned int *lenp)
    if (str == NULL)
      return 0;
 
+   if (trim_trailing)
+     {
+	unsigned int len1 = len;
+	while (len1)
+	  {
+	     len1--;
+	     if (0 == isspace(str[len1]))
+	       {
+		  len1++;
+		  break;
+	       }
+	  }
+	len = len1;
+     }
+	       
    *strp = SLang_create_nslstring (str, len);
    if (str != buf) SLfree (str);
 
@@ -373,7 +390,7 @@ static int stdio_fgets (SLang_Ref_Type *ref, SL_File_Table_Type *t)
    if (NULL == (fp = check_fp (t, SL_READ)))
      return -1;
 
-   status = read_one_line (fp, &s, &len);
+   status = read_one_line (fp, &s, &len, 0);
    if (status <= 0)
      return -1;
 
@@ -413,7 +430,7 @@ static void stdio_fgetslines_internal (FILE *fp, unsigned int n)
 	char *line;
 	unsigned int len;
 
-	status = read_one_line (fp, &line, &len);
+	status = read_one_line (fp, &line, &len, 0);
 	if (status == -1)
 	  goto return_error;
 
@@ -872,8 +889,9 @@ struct _SLang_Foreach_Context_Type
 {
    SLang_MMT_Type *mmt;
    FILE *fp;
-#define CTX_USE_LINE 1
-#define CTX_USE_CHAR 2
+#define CTX_USE_LINE		1
+#define CTX_USE_CHAR		2
+#define CTX_USE_LINE_WS		3
    unsigned char type;
 };
 
@@ -908,6 +926,8 @@ cl_foreach_open (unsigned char type, unsigned int num)
 	  type = CTX_USE_CHAR;
 	else if (0 == strcmp (s, "line"))
 	  type = CTX_USE_LINE;
+	else if (0 == strcmp (s, "wsline"))
+	  type = CTX_USE_LINE_WS;
 	else
 	  {
 	     SLang_verror (SL_NOT_IMPLEMENTED,
@@ -972,7 +992,8 @@ static int cl_foreach (unsigned char type, SLang_Foreach_Context_Type *c)
 	return 1;
 
       case CTX_USE_LINE:
-	status = read_one_line (c->fp, &s, &len);
+      case CTX_USE_LINE_WS:
+	status = read_one_line (c->fp, &s, &len, (c->type==CTX_USE_LINE_WS));
 	if (status <= 0)
 	  return status;
 	if (0 == _SLang_push_slstring (s))
