@@ -40,18 +40,15 @@
 
 /*{{{ Utility Functions */
 
-static char Utility_Char_Table [256];
+static unsigned char Utility_Char_Table [256];
 static unsigned char WhiteSpace_Lut[256];
 
 static void set_utility_char_table (char *pos) /*{{{*/
 {
-   register char *t = Utility_Char_Table, *tmax;
+   register unsigned char *t = Utility_Char_Table;
    register unsigned char ch;
 
-   tmax = t + 256;
-   while (t < tmax) *t++ = 0;
-
-   t = Utility_Char_Table;
+   memset ((char *) t, 0, sizeof (Utility_Char_Table));
    while ((ch = (unsigned char) *pos++) != 0) t[ch] = 1;
 }
 
@@ -88,20 +85,27 @@ static unsigned int do_trim (char **beg, int do_beg,
 {
    unsigned int len;
    char *a, *b;
+   unsigned char *lut;
 
-   set_utility_char_table (white);
+   if (white == NULL)
+     lut = make_whitespace_lut ();
+   else
+     {
+	set_utility_char_table (white);
+	lut = Utility_Char_Table;
+     }
 
    a = *beg;
    len = strlen (a);
    b = a + len;
 
    if (do_beg)
-     while (Utility_Char_Table[(unsigned char) *a]) a++;
+     while (lut[(unsigned char) *a]) a++;
 
    if (do_end)
      {
 	b--;
-	while ((b >= a) && (Utility_Char_Table[(unsigned char) *b])) b--;
+	while ((b >= a) && (lut[(unsigned char) *b])) b--;
 	b++;
      }
 
@@ -240,7 +244,7 @@ static void strtrim_cmd_internal (char *str, int do_beg, int do_end)
 	  return;
 	free_str = 1;
      }
-   else white = " \t\f\r\n";
+   else white = NULL;
 
    beg = str;
    len = do_trim (&beg, do_beg, &end, do_end, white);
@@ -576,8 +580,9 @@ static void str_quote_string_cmd (char *str, char *quotes, int *slash_ptr) /*{{{
    char *q;
    int slash;
    unsigned int len;
-   register char *t, *s, *q1;
-   register unsigned char ch;
+   char *s, *q1;
+   unsigned char ch;
+   unsigned char *t;
 
    slash = *slash_ptr;
 
@@ -1116,7 +1121,7 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     /* Pointer type?? Why?? */
 	     if (-1 == SLdo_pop ())
 	       return out;
-	     varp = (VOID_STAR) _SLStack_Pointer;
+	     varp = (VOID_STAR) _SLang_get_run_stack_pointer ();
 	     use_varp = 1;
 	     use_long = 0;
 	     break;
@@ -1180,7 +1185,7 @@ int _SLstrops_do_sprintf_n (int n) /*{{{*/
    if (-1 == (ofs = SLreverse_stack (n + 1)))
      return -1;
 
-   ptr = _SLRun_Stack + ofs;
+   ptr = _SLang_get_run_stack_base () + ofs;
 
    if (SLang_pop_slstring(&fmt))
      return -1;
@@ -1188,8 +1193,7 @@ int _SLstrops_do_sprintf_n (int n) /*{{{*/
    p = SLdo_sprintf (fmt);
    SLang_free_slstring (fmt);
 
-   while (_SLStack_Pointer > ptr)
-     SLdo_pop ();
+   SLdo_pop_n (_SLang_get_run_stack_pointer () - ptr);
 
    if (SLang_Error)
      {
