@@ -147,27 +147,29 @@ static _SLstruct_Field_Type *find_field (_SLang_Struct_Type *s, char *name)
    return NULL;
 }
 
-static _SLstruct_Field_Type *pop_field (_SLang_Struct_Type *s, char *name)
+static _SLstruct_Field_Type *pop_field (_SLang_Struct_Type *s, char *name,
+					_SLstruct_Field_Type *(*find)(_SLang_Struct_Type *, char *))
 {
    _SLstruct_Field_Type *f;
 
-   f = find_field (s, name);
+   f = (*find) (s, name);
    if (f == NULL)
-     SLang_verror (SL_SYNTAX_ERROR, "struct has no field named %s", name);
+     SLang_verror (SL_INVALID_PARM, "struct has no field named %s", name);
    return f;
 }
 
-int SLstruct_create_struct (unsigned int nfields,
-			    char **field_names,
-			    unsigned char *field_types,
-			    VOID_STAR *field_values)
+static _SLang_Struct_Type *
+  create_struct (unsigned int nfields,
+		 char **field_names,
+		 unsigned char *field_types,
+		 VOID_STAR *field_values)
 {
    _SLang_Struct_Type *s;
    _SLstruct_Field_Type *f;
    unsigned int i;
 
    if (NULL == (s = allocate_struct (nfields)))
-     return -1;
+     return NULL;
 
    f = s->fields;
    for (i = 0; i < nfields; i++)
@@ -179,7 +181,7 @@ int SLstruct_create_struct (unsigned int nfields,
 
 	if (name == NULL)
 	  {
-	     SLang_verror (SL_INVALID_PARM, "A struct field name cannot be NULL");
+	     SLang_verror (SL_APPLICATION_ERROR, "A struct field name cannot be NULL");
 	     goto return_error;
 	  }
 
@@ -199,18 +201,33 @@ int SLstruct_create_struct (unsigned int nfields,
 	type = field_types[i];
 	cl = _SLclass_get_class (type);
 
-	if ((-1 == (cl->cl_push (type, value)))
+	if ((-1 == (cl->cl_apush (type, value)))
 	    || (-1 == SLang_pop (&f->obj)))
 	  goto return_error;
 
 	f++;
      }
 
-   if (0 == _SLang_push_struct (s))
-     return 0;
-   /* drop */
+   return s;
 
    return_error:
+   _SLstruct_delete_struct (s);
+   return NULL;
+}
+
+int SLstruct_create_struct (unsigned int nfields,
+			    char **field_names,
+			    unsigned char *field_types,
+			    VOID_STAR *field_values)
+{
+   _SLang_Struct_Type *s;
+   
+   if (NULL == (s = create_struct (nfields, field_names, field_types, field_values)))
+     return -1;
+
+   if (0 == _SLang_push_struct (s))
+     return 0;
+
    _SLstruct_delete_struct (s);
    return -1;
 }
@@ -493,7 +510,7 @@ static int struct_sput (unsigned char type, char *name)
    if (-1 == _SLang_pop_struct (&s))
      return -1;
 
-   if ((NULL == (f = pop_field (s, name)))
+   if ((NULL == (f = pop_field (s, name, find_field)))
        || (-1 == SLang_pop (&obj)))
      {
 	_SLstruct_delete_struct (s);
@@ -517,7 +534,7 @@ static int struct_sget (unsigned char type, char *name)
    if (-1 == _SLang_pop_struct (&s))
      return -1;
 
-   if (NULL == (f = pop_field (s, name)))
+   if (NULL == (f = pop_field (s, name, find_field)))
      {
 	_SLstruct_delete_struct (s);
 	return -1;
@@ -534,7 +551,7 @@ static int struct_typecast
 {
    _SLang_Struct_Type **a, **b;
    unsigned int i;
-   
+
    (void) a_type;
    (void) b_type;
 
@@ -604,7 +621,7 @@ int _SLstruct_define_typedef (void)
 	/* There is a memory leak here if this fails... */
 	return -1;
      }
-   /* Note: typecast from a user type struct type allowed but not the other
+   /* Note: typecast from a user type to a struct type allowed but not the other
     * way.
     */
    if (-1 == SLclass_add_typecast (cl->cl_data_type, SLANG_STRUCT_TYPE, struct_typecast, 1))
@@ -617,7 +634,7 @@ static int
 struct_datatype_deref (unsigned char stype)
 {
    (void) stype;
-   
+
    if (SLang_peek_at_stack () == SLANG_ARRAY_TYPE)
      {
 	SLang_Array_Type *at;
@@ -625,7 +642,7 @@ struct_datatype_deref (unsigned char stype)
 
 	if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
 	  return -1;
-	
+
 	status = SLstruct_create_struct (at->num_elements,
 					 (char **) at->data, NULL, NULL);
 
@@ -734,7 +751,7 @@ static void struct_set_field (void)
 	return;
      }
 
-   if (NULL == (f = pop_field (s, name)))
+   if (NULL == (f = pop_field (s, name, find_field)))
      {
 	_SLstruct_delete_struct (s);
 	SLang_free_slstring (name);
@@ -807,7 +824,7 @@ static int is_struct_type (void)
 
    if (-1 == SLang_pop (&obj))
      return -1;
-   
+
    type = obj.data_type;
    if (type == SLANG_STRUCT_TYPE)
      status = 1;
@@ -815,9 +832,8 @@ static int is_struct_type (void)
      status = (NULL != _SLclass_get_class (type)->cl_struct_def);
    SLang_free_object (&obj);
    return status;
-}	
-   
-     
+}
+
 static SLang_Intrin_Fun_Type Struct_Table [] =
 {
    MAKE_INTRINSIC_1("get_struct_field_names", get_struct_field_names, SLANG_VOID_TYPE, SLANG_STRUCT_TYPE),
@@ -933,3 +949,164 @@ void _SLstruct_push_args (SLang_Array_Type *at)
 	(void) _SLpush_slang_obj (&s->fields->obj);
      }
 }
+
+/* C structures */
+static _SLstruct_Field_Type *find_field_via_strcmp (_SLang_Struct_Type *s, char *name)
+{
+   _SLstruct_Field_Type *f, *fmax;
+
+   f = s->fields;
+   fmax = f + s->nfields;
+
+   while (f < fmax)
+     {
+	if (0 == strcmp (name, f->name))
+	  return f;
+
+	f++;
+     }
+   return NULL;
+}
+
+static void free_cstruct_field (SLang_CStruct_Field_Type *cfield, char *cs)
+{
+   SLang_Class_Type *cl;
+
+   if ((cfield->read_only == 0) 
+       && (NULL != (cl = _SLclass_get_class (cfield->type))))
+     _SLarray_free_array_elements (cl, (VOID_STAR)(cs + cfield->offset), 1);
+}
+
+void SLang_free_cstruct (VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
+{
+   if ((cs == NULL) || (cfields == NULL))
+     return;
+
+   while (cfields->field_name != NULL)
+     {
+	free_cstruct_field (cfields, (char *) cs);
+	cfields++;
+     }
+}
+
+int SLang_pop_cstruct (VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
+{
+   _SLang_Struct_Type *s;
+   SLang_CStruct_Field_Type *cfield;
+   char *field_name;
+   char *cs_addr;
+
+   if ((cfields == NULL) || (cs == NULL))
+     return -1;
+
+   if (-1 == _SLang_pop_struct (&s))
+     return -1;
+
+   cfield = cfields;
+   cs_addr = (char *) cs;
+
+   while (NULL != (field_name = cfield->field_name))
+     {
+	_SLstruct_Field_Type *f;
+	SLang_Class_Type *cl;
+
+	if ((cfield->read_only == 0)
+	    && ((NULL == (f = pop_field (s, field_name, find_field_via_strcmp)))
+		|| (-1 == _SLpush_slang_obj (&f->obj))
+		|| (NULL == (cl = _SLclass_get_class (cfield->type)))
+		|| (-1 == (*cl->cl_apop)(cfield->type, (VOID_STAR) (cs_addr + cfield->offset)))))
+	  goto return_error;
+
+	cfield++;
+     }
+
+   _SLstruct_delete_struct (s);
+   return 0;
+
+   return_error:
+   while (cfield != cfields)
+     {
+	free_cstruct_field (cfield, cs);
+	cfield--;
+     }
+   _SLstruct_delete_struct (s);
+   return -1;
+}
+
+static _SLang_Struct_Type *create_cstruct (VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
+{
+   unsigned int i, n;
+   _SLang_Struct_Type *s;
+   SLang_CStruct_Field_Type *cfield;
+   char **field_names;
+   VOID_STAR *field_values;
+   SLtype *field_types;
+
+   if ((cs == NULL) || (cfields == NULL))
+     return NULL;
+
+   cfield = cfields;
+   while (cfield->field_name != NULL)
+     cfield++;
+   n = cfield - cfields;
+   if (n == 0)
+     {
+	SLang_verror (SL_APPLICATION_ERROR, "C structure has no fields");
+	return NULL;
+     }
+   
+   s = NULL;
+   field_types = NULL;
+   field_values = NULL;
+   if ((NULL == (field_names = (char **) SLmalloc (n*sizeof (char *))))
+       || (NULL == (field_types = (SLtype *)SLmalloc (n*sizeof(SLtype))))
+       || (NULL == (field_values = (VOID_STAR *)SLmalloc (n*sizeof(VOID_STAR)))))
+     goto return_error;
+
+   for (i = 0; i < n; i++)
+     {
+	cfield = cfields + i;
+	field_names[i] = cfield->field_name;
+	field_types[i] = cfield->type;
+	field_values[i] = (VOID_STAR)((char *)cs + cfield->offset);
+     }
+
+   s = create_struct (n, field_names, field_types, field_values);
+   /* drop */
+
+   return_error:
+   SLfree ((char *) field_values);
+   SLfree ((char *) field_types);
+   SLfree ((char *) field_names);
+
+   return s;
+}
+   
+int SLang_push_cstruct (VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
+{
+   _SLang_Struct_Type *s;
+   
+   if (NULL == (s = create_cstruct (cs, cfields)))
+     return -1;
+   
+   if (0 == _SLang_push_struct (s))
+     return 0;
+   
+   _SLstruct_delete_struct (s);
+   return -1;
+}
+
+int SLang_assign_cstruct_to_ref (SLang_Ref_Type *ref, VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
+{
+   _SLang_Struct_Type *s;
+
+   if (NULL == (s = create_cstruct (cs, cfields)))
+     return -1;
+
+   if (0 == SLang_assign_to_ref (ref, SLANG_STRUCT_TYPE, (VOID_STAR) &s))
+     return 0;
+
+   _SLstruct_delete_struct (s);
+   return -1;
+}
+
