@@ -171,37 +171,25 @@ char *SLcurrent_time_string (void) /*{{{*/
 
 /*}}}*/
 
+static SLang_CStruct_Field_Type TM_Struct [] =
+{
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_sec, "tm_sec", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_min, "tm_min", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_hour, "tm_hour", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_mday, "tm_mday", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_mon, "tm_mon", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_year, "tm_year", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_wday, "tm_wday", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_yday, "tm_yday", 0),
+   MAKE_CSTRUCT_INT_FIELD(struct tm, tm_isdst, "tm_isdst", 0),
+   SLANG_END_CSTRUCT_TABLE
+};
+
 static int push_tm_struct (struct tm *tms)
 {
-   char *field_names [9];
-   unsigned char field_types[9];
-   VOID_STAR field_values [9];
-   int int_values [9];
-   unsigned int i;
-
-   if (tms == NULL)
-     return SLang_push_null ();
-
-   field_names [0] = "tm_sec"; int_values [0] = tms->tm_sec;
-   field_names [1] = "tm_min"; int_values [1] = tms->tm_min;
-   field_names [2] = "tm_hour"; int_values [2] = tms->tm_hour;
-   field_names [3] = "tm_mday"; int_values [3] = tms->tm_mday;
-   field_names [4] = "tm_mon"; int_values [4] = tms->tm_mon;
-   field_names [5] = "tm_year"; int_values [5] = tms->tm_year;
-   field_names [6] = "tm_wday"; int_values [6] = tms->tm_wday;
-   field_names [7] = "tm_yday"; int_values [7] = tms->tm_yday;
-   field_names [8] = "tm_isdst"; int_values [8] = tms->tm_isdst;
-
-   for (i = 0; i < 9; i++)
-     {
-	field_types [i] = SLANG_INT_TYPE;
-	field_values [i] = (VOID_STAR) (int_values + i);
-     }
-
-   return SLstruct_create_struct (9, field_names, field_types, field_values);
+   return SLang_push_cstruct ((VOID_STAR) tms, TM_Struct);
 }
 
-
 static void localtime_cmd (long *t)
 {
    time_t tt = (time_t) *t;
@@ -236,29 +224,36 @@ static void gmtime_cmd (long *t)
 # endif
 #endif
 
+typedef struct
+{ 
+   double tms_utime;
+   double tms_stime;
+   double tms_cutime;
+   double tms_cstime;
+}
+TMS_Type;
+
+static SLang_CStruct_Field_Type TMS_Struct [] =
+{
+   MAKE_CSTRUCT_FIELD(TMS_Type, tms_utime, "tms_utime", SLANG_DOUBLE_TYPE, 0),
+   MAKE_CSTRUCT_FIELD(TMS_Type, tms_utime, "tms_stime", SLANG_DOUBLE_TYPE, 0),
+   MAKE_CSTRUCT_FIELD(TMS_Type, tms_utime, "tms_cutime", SLANG_DOUBLE_TYPE, 0),
+   MAKE_CSTRUCT_FIELD(TMS_Type, tms_utime, "tms_cstime", SLANG_DOUBLE_TYPE, 0),
+   SLANG_END_CSTRUCT_TABLE
+};
+
 static void times_cmd (void)
 {
-   double dvals[4];
+   TMS_Type d;
    struct tms t;
-   VOID_STAR field_values[4];
-   char *field_names[4];
-   unsigned int i;
-   unsigned char field_types[4];
-   
+
    (void) times (&t);
 
-   field_names[0] = "tms_utime";   dvals[0] = (double)t.tms_utime; 
-   field_names[1] = "tms_stime";   dvals[1] = (double)t.tms_stime; 
-   field_names[2] = "tms_cutime";  dvals[2] = (double)t.tms_cutime;
-   field_names[3] = "tms_cstime";  dvals[3] = (double)t.tms_cstime;
-
-   for (i = 0; i < 4; i++)
-     {
-	dvals[i] *= SECS_PER_TICK;
-	field_values[i] = (VOID_STAR) &dvals[i];
-	field_types[i] = SLANG_DOUBLE_TYPE;
-     }
-   (void) SLstruct_create_struct (4, field_names, field_types, field_values);
+   d.tms_utime = SECS_PER_TICK * (double)t.tms_utime; 
+   d.tms_stime = SECS_PER_TICK * (double)t.tms_stime;
+   d.tms_cutime = SECS_PER_TICK * (double)t.tms_cutime;
+   d.tms_cstime = SECS_PER_TICK * (double)t.tms_cstime;
+   (void) SLang_push_cstruct ((VOID_STAR)&d, TMS_Struct);
 }
 
 static struct tms Tic_TMS;
