@@ -151,7 +151,7 @@ static void do_line_file_error (int line, char *file)
  * The third character null terminates the operator string.  This is for
  * the token structure.
  */
-static char Operators [29][4] =
+static SLCONST char Operators [29][4] =
 {
 #define OFS_EXCL	0
      {'!',	'=',	0, NE_TOKEN},
@@ -198,7 +198,7 @@ static char Operators [29][4] =
      {	0,	0,	0, EOF_TOKEN}
 };
 
-static unsigned char Char_Type_Table[256][2] =
+static SLCONST unsigned char Char_Type_Table[256][2] =
 {
  { NL_CHAR, 0 },	/* 0x0 */   { BAD_CHAR, 0 },	/* 0x1 */
  { BAD_CHAR, 0 },	/* 0x2 */   { BAD_CHAR, 0 },	/* 0x3 */
@@ -541,7 +541,7 @@ static int get_op_token (_SLang_Token_Type *tok, char ch)
    unsigned int offset;
    char second_char;
    unsigned char type;
-   char *name;
+   SLCONST char *name;
 
    /* operators are: + - / * ++ -- += -= = == != > < >= <= | etc..
     * These lex to the longest valid operator token.
@@ -580,7 +580,7 @@ static int get_op_token (_SLang_Token_Type *tok, char ch)
 	return type;
      }
 
-   tok->v.s_val = name;
+   tok->v.s_val = (char *)name;
 
    if (name[1] == 0)
      unget_prep_char (second_char);
