@@ -556,7 +556,7 @@ static int ref_push (unsigned char type, VOID_STAR ptr)
    return _SLang_push_ref (ref->is_global, (VOID_STAR) ref->v.nt);
 }
 
-int SLang_assign_to_ref (SLang_Ref_Type *ref, unsigned char type, VOID_STAR v)
+int SLang_assign_to_ref (SLang_Ref_Type *ref, SLtype type, VOID_STAR v)
 {
    SLang_Object_Type *stkptr;
    SLang_Class_Type *cl;
@@ -582,11 +582,11 @@ int SLang_assign_to_ref (SLang_Ref_Type *ref, unsigned char type, VOID_STAR v)
    if (-1 == (*cl->cl_apush) (type, v))
      return -1;
 
-   stkptr = _SLStack_Pointer;
+   stkptr = _SLang_get_run_stack_pointer ();
    if (0 == _SLang_deref_assign (ref))
      return 0;
 
-   if (stkptr != _SLStack_Pointer)
+   if (stkptr != _SLang_get_run_stack_pointer ())
      SLdo_pop ();
 
    return -1;
